@@ -79,7 +79,7 @@ class PostgreSQLOfflineStore(OfflineStore):
                 SELECT {a_field_string},
                 ROW_NUMBER() OVER({partition_by_join_key_string} ORDER BY {timestamp_desc_string}) AS _feast_row
                 FROM ({from_expression}) a
-                WHERE a.{event_timestamp_column} BETWEEN '{start_date}'::timestamptz AND '{end_date}'::timestamptz
+                WHERE a."{event_timestamp_column}" BETWEEN '{start_date}'::timestamptz AND '{end_date}'::timestamptz
             ) b
             WHERE _feast_row = 1
             """
@@ -132,6 +132,13 @@ class PostgreSQLOfflineStore(OfflineStore):
                 feature_refs, feature_views, registry, project,
             )
 
+            query_context = [asdict(context) for context in query_context]
+            # Hack for query_context.entity_selections to support uppercase in columns
+            for context in query_context:
+                context["entity_selections"] = [
+                    f'''"{entity_selection.replace(' AS ', '" AS "')}\"'''
+                    for entity_selection in context["entity_selections"]
+                ]
             try:
                 yield build_point_in_time_query(
                     query_context,
@@ -229,22 +236,22 @@ def _append_alias(field_names: List[str], alias: str) -> List[str]:
 
 
 def build_point_in_time_query(
-    feature_view_query_contexts: List[offline_utils.FeatureViewQueryContext],
+    feature_view_query_contexts: List[dict],
     left_table_query_string: str,
     entity_df_event_timestamp_col: str,
     entity_df_columns: KeysView[str],
     query_template: str,
     full_feature_names: bool = False,
 ) -> str:
-    """Build point-in-time query between each feature view table and the entity dataframe for Bigquery and Redshift"""
+    """Build point-in-time query between each feature view table and the entity dataframe for PostgreSQL"""
     template = Environment(loader=BaseLoader()).from_string(source=query_template)
 
     final_output_feature_names = list(entity_df_columns)
     final_output_feature_names.extend(
         [
-            (f"{fv.name}__{feature}" if full_feature_names else feature)
+            (f'{fv["name"]}__{feature}' if full_feature_names else feature)
             for fv in feature_view_query_contexts
-            for feature in fv.features
+            for feature in fv["features"]
         ]
     )
 
@@ -253,9 +260,9 @@ def build_point_in_time_query(
         "left_table_query_string": left_table_query_string,
         "entity_df_event_timestamp_col": entity_df_event_timestamp_col,
         "unique_entity_keys": set(
-            [entity for fv in feature_view_query_contexts for entity in fv.entities]
+            [entity for fv in feature_view_query_contexts for entity in fv["entities"]]
         ),
-        "featureviews": [asdict(context) for context in feature_view_query_contexts],
+        "featureviews": feature_view_query_contexts,
         "full_feature_names": full_feature_names,
         "final_output_feature_names": final_output_feature_names,
     }
@@ -281,12 +288,12 @@ WITH entity_dataframe AS (
             {% if featureview.entities %}
             ,(
                 {% for entity in featureview.entities %}
-                    CAST({{entity}} as VARCHAR) ||
+                    CAST("{{entity}}" as VARCHAR) ||
                 {% endfor %}
-                CAST({{entity_df_event_timestamp_col}} AS VARCHAR)
-            ) AS {{featureview.name}}__entity_row_unique_id
+                CAST("{{entity_df_event_timestamp_col}}" AS VARCHAR)
+            ) AS "{{featureview.name}}__entity_row_unique_id"
             {% else %}
-            ,CAST({{entity_df_event_timestamp_col}} AS VARCHAR) AS {{featureview.name}}__entity_row_unique_id
+            ,CAST("{{entity_df_event_timestamp_col}}" AS VARCHAR) AS "{{featureview.name}}__entity_row_unique_id"
             {% endif %}
         {% endfor %}
     FROM {{ left_table_query_string }}
@@ -294,16 +301,16 @@ WITH entity_dataframe AS (
 
 {% for featureview in featureviews %}
 
-{{ featureview.name }}__entity_dataframe AS (
+"{{ featureview.name }}__entity_dataframe" AS (
     SELECT
-        {{ featureview.entities | join(', ')}}{% if featureview.entities %},{% else %}{% endif %}
+        {% if featureview.entities %}"{{ featureview.entities | join('", "') }}",{% endif %}
         entity_timestamp,
-        {{featureview.name}}__entity_row_unique_id
+        "{{featureview.name}}__entity_row_unique_id"
     FROM entity_dataframe
     GROUP BY
-        {{ featureview.entities | join(', ')}}{% if featureview.entities %},{% else %}{% endif %}
+        {% if featureview.entities %}"{{ featureview.entities | join('", "')}}",{% endif %}
         entity_timestamp,
-        {{featureview.name}}__entity_row_unique_id
+        "{{featureview.name}}__entity_row_unique_id"
 ),
 
 /*
@@ -323,28 +330,28 @@ WITH entity_dataframe AS (
  of the data that is not relevant.
 */
 
-{{ featureview.name }}__subquery AS (
+"{{ featureview.name }}__subquery" AS (
     SELECT
-        {{ featureview.event_timestamp_column }} as event_timestamp,
-        {{ featureview.created_timestamp_column ~ ' as created_timestamp,' if featureview.created_timestamp_column else '' }}
+        "{{ featureview.event_timestamp_column }}" as event_timestamp,
+        {{ '"' ~ featureview.created_timestamp_column ~ '" as created_timestamp,' if featureview.created_timestamp_column else '' }}
         {{ featureview.entity_selections | join(', ')}}{% if featureview.entity_selections %},{% else %}{% endif %}
         {% for feature in featureview.features %}
             "{{ feature }}" as "{% if full_feature_names %}{{ featureview.name }}__{{feature}}{% else %}{{ feature }}{% endif %}"{% if loop.last %}{% else %}, {% endif %}
         {% endfor %}
     FROM {{ featureview.table_subquery }} AS sub
-    WHERE {{ featureview.event_timestamp_column }} <= (SELECT MAX(entity_timestamp) FROM entity_dataframe)
+    WHERE "{{ featureview.event_timestamp_column }}" <= (SELECT MAX(entity_timestamp) FROM entity_dataframe)
     {% if featureview.ttl == 0 %}{% else %}
-    AND {{ featureview.event_timestamp_column }} >= (SELECT MIN(entity_timestamp) FROM entity_dataframe) - {{ featureview.ttl }} * interval '1' second
+    AND "{{ featureview.event_timestamp_column }}" >= (SELECT MIN(entity_timestamp) FROM entity_dataframe) - {{ featureview.ttl }} * interval '1' second
     {% endif %}
 ),
 
-{{ featureview.name }}__base AS (
+"{{ featureview.name }}__base" AS (
     SELECT
         subquery.*,
         entity_dataframe.entity_timestamp,
-        entity_dataframe.{{featureview.name}}__entity_row_unique_id
-    FROM {{ featureview.name }}__subquery AS subquery
-    INNER JOIN {{ featureview.name }}__entity_dataframe AS entity_dataframe
+        entity_dataframe."{{featureview.name}}__entity_row_unique_id"
+    FROM "{{ featureview.name }}__subquery" AS subquery
+    INNER JOIN "{{ featureview.name }}__entity_dataframe" AS entity_dataframe
     ON TRUE
         AND subquery.event_timestamp <= entity_dataframe.entity_timestamp
 
@@ -353,7 +360,7 @@ WITH entity_dataframe AS (
         {% endif %}
 
         {% for entity in featureview.entities %}
-        AND subquery.{{ entity }} = entity_dataframe.{{ entity }}
+        AND subquery."{{ entity }}" = entity_dataframe."{{ entity }}"
         {% endfor %}
 ),
 
@@ -364,13 +371,13 @@ WITH entity_dataframe AS (
  We then join the data on the next CTE
 */
 {% if featureview.created_timestamp_column %}
-{{ featureview.name }}__dedup AS (
+"{{ featureview.name }}__dedup" AS (
     SELECT
-        {{featureview.name}}__entity_row_unique_id,
+        "{{featureview.name}}__entity_row_unique_id",
         event_timestamp,
         MAX(created_timestamp) as created_timestamp
-    FROM {{ featureview.name }}__base
-    GROUP BY {{featureview.name}}__entity_row_unique_id, event_timestamp
+    FROM "{{ featureview.name }}__base"
+    GROUP BY "{{featureview.name}}__entity_row_unique_id", event_timestamp
 ),
 {% endif %}
 
@@ -378,22 +385,22 @@ WITH entity_dataframe AS (
  3. The data has been filtered during the first CTE "*__base"
  Thus we only need to compute the latest timestamp of each feature.
 */
-{{ featureview.name }}__latest AS (
+"{{ featureview.name }}__latest" AS (
     SELECT
         event_timestamp,
         {% if featureview.created_timestamp_column %}created_timestamp,{% endif %}
-        {{featureview.name}}__entity_row_unique_id
+        "{{featureview.name}}__entity_row_unique_id"
     FROM
     (
         SELECT *,
             ROW_NUMBER() OVER(
-                PARTITION BY {{featureview.name}}__entity_row_unique_id
+                PARTITION BY "{{featureview.name}}__entity_row_unique_id"
                 ORDER BY event_timestamp DESC{% if featureview.created_timestamp_column %},created_timestamp DESC{% endif %}
             ) AS row_number
-        FROM {{ featureview.name }}__base
+        FROM "{{ featureview.name }}__base"
         {% if featureview.created_timestamp_column %}
-            INNER JOIN {{ featureview.name }}__dedup
-            USING ({{featureview.name}}__entity_row_unique_id, event_timestamp, created_timestamp)
+            INNER JOIN "{{ featureview.name }}__dedup"
+            USING ("{{featureview.name}}__entity_row_unique_id", event_timestamp, created_timestamp)
         {% endif %}
     ) AS sub
     WHERE row_number = 1
@@ -403,12 +410,12 @@ WITH entity_dataframe AS (
  4. Once we know the latest value of each feature for a given timestamp,
  we can join again the data back to the original "base" dataset
 */
-{{ featureview.name }}__cleaned AS (
+"{{ featureview.name }}__cleaned" AS (
     SELECT base.*
-    FROM {{ featureview.name }}__base as base
-    INNER JOIN {{ featureview.name }}__latest
+    FROM "{{ featureview.name }}__base" as base
+    INNER JOIN "{{ featureview.name }}__latest"
     USING(
-        {{featureview.name}}__entity_row_unique_id,
+        "{{featureview.name}}__entity_row_unique_id",
         event_timestamp
         {% if featureview.created_timestamp_column %}
             ,created_timestamp
@@ -432,7 +439,7 @@ LEFT JOIN (
         {% for feature in featureview.features %}
             ,"{% if full_feature_names %}{{ featureview.name }}__{{feature}}{% else %}{{ feature }}{% endif %}"
         {% endfor %}
-    FROM {{ featureview.name }}__cleaned
-) AS "{{featureview.name}}" USING ({{featureview.name}}__entity_row_unique_id)
+    FROM "{{ featureview.name }}__cleaned"
+) AS "{{featureview.name}}" USING ("{{featureview.name}}__entity_row_unique_id")
 {% endfor %}
 """
